window.Gleam.initSearch([{"doc":"glelm","title":"glelm","content":"# glelm\n\nElm-to-Gleam transpiler.\n\nWORK-IN-PROGRESS.\n\nCurrently supported:\n* Naive transpilling of type declarations (e.g. `Maybe` is `Maybe` in generated Gleam code).\n* Sum types\n* Typed Records\n* Function types\n* Tuples\n* Unit\n\nComing up soon:\n* Type aliases `type alias A = B`\n* Untyped records  `type alias R = { foo: Int }`\n\nUnsupported:\n* Function declarations\n* Ports\n\n\n[![Package Version](https://img.shields.io/hexpm/v/glelm)](https://hex.pm/packages/glelm)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glelm/)\n\n```sh\ncat samples/Sample1.elm\n```\n\n```elm\nmodule Sample1 exposing (Declaration(..), Node)\n\n\n-- Elm declaration\nmodule Sample1 exposing (Declaration(..), Node)\n\n{-| Elm declaration\n-}\ntype Declaration\n    = FunctionDeclaration Function\n    | AliasDeclaration TypeAlias\n    | CustomTypeDeclaration Type\n    | PortDeclaration Signature\n    | InfixDeclaration Infix\n    | Destructuring\n        { pattern : Node Pattern\n        , expression : Node Expression\n        }\n\n\n{-| Provides additional context\n-}\ntype Node a\n    = Node Range a\n```\n\n``` sh\ngleam run samples/Sample1.elm\n[...]\n```\n\n```gleam\npub type Declaration {\n  FunctionDeclaration(Function)\n  AliasDeclaration(TypeAlias)\n  CustomTypeDeclaration(Type)\n  PortDeclaration(Signature)\n  InfixDeclaration(Infix)\n  Destructuring(pattern: Node(Pattern), expression: Node(Expression))\n}\n\npub opaque type Node(a) {\n  Node(Range, a)\n}\nSuccess!\n```\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n\n### Running elm-syntax\n\nThere is a web app you can use to turn Elm source code into AST compatible with\n[elm-syntax](https://package.elm-lang.org/packages/stil4m/elm-syntax/7.3.8/)\nwhich is an inspiration for Elm AST in this project. The generated JSON isn&#39;t\nprefect but it&#39;s close enough to help figuring out some quirkier aspects.\n\n``` sh\ncd tools/elm-syntax\nelm reactor\n```\n\n### Contributing: supporting more syntax\n\nThe flow for adding support for more Elm syntax is as follows:\n\n1. Add a test case to `test/transpile` by adding two files, `xyz.elm` containing\n   the input and `xyz.expected`.\n2. If necessary, use `tools/elm-syntax` to inspect the corresponding elm-syntax\n   AST.\n3. If you need to add new tokens, add them in `elm/lexer.gleam`. Don&#39;t forget\n   about tests.\n4. Implement the parser in `elm/parser.gleam`, adding tests where necessary.\n5. Update the transpiler in `transpile.gleam`.\n6. Run tests via `gleam test`. Iterate until they pass.\n","url":"index.html"},{"doc":"glelm/debug","title":"enabled","content":"pub fn enabled() -&gt; Bool\n\nSynonyms:\nglelm/debug.enabled\nglelm/debug enabled","url":"glelm/debug.html#enabled"},{"doc":"glelm/debug","title":"log","content":"pub fn log(x: a) -&gt; a\n\nSynonyms:\nglelm/debug.log\nglelm/debug log","url":"glelm/debug.html#log"},{"doc":"glelm/debug","title":"glelm/debug","content":"","url":"glelm/debug.html"},{"doc":"glelm/elm/lexer","title":"Error","content":"pub type Error =\n  lexer.Error\n\n\nSynonyms:\nglelm/elm/lexer.Error\nglelm/elm/lexer Error","url":"glelm/elm/lexer.html#Error"},{"doc":"glelm/elm/lexer","title":"Lexer","content":"pub opaque type Lexer\n\n\nSynonyms:\nglelm/elm/lexer.Lexer\nglelm/elm/lexer Lexer","url":"glelm/elm/lexer.html#Lexer"},{"doc":"glelm/elm/lexer","title":"Token","content":"pub type Token {\n  TypeKeyword\n  UnitKeyword\n  UpcaseIdentifier(String)\n  VerticalBar\n  Eq\n  LParen\n  RParen\n  Comma\n  Comment(String)\n  LetKeyword\n  InKeyword\n  CaseKeyword\n  OfKeyword\n  WhereKeyword\n  DoKeyword\n  TypeAliasKeyword\n  PortModuleKeyword\n  ModuleKeyword\n  IfKeyword\n  ThenKeyword\n  ElseKeyword\n  ImportKeyword\n  ExposingKeyword\n  AsKeyword\n  BoolLiteral(Bool)\n  FloatLiteral(Float)\n  IntLiteral(Int)\n  StringLiteral(String)\n  Identifier(String)\n  Arrow\n  Pipeline\n  ReverseApply\n  Cons\n  Concat\n  Plus\n  Minus\n  Multiply\n  Divide\n  IntDivide\n  Power\n  Modulo\n  Equal\n  NotEqual\n  LessThan\n  GreaterThan\n  LessEqual\n  GreaterEqual\n  LBracket\n  RBracket\n  LBrace\n  RBrace\n  Dot\n  DoubleDot\n  Colon\n  LayoutStart\n  LayoutEnd\n  LayoutSemicolon\n}\n\nTypeKeyword\n\n\nUnitKeyword\n\n\nUpcaseIdentifier(String)\n\n\nVerticalBar\n\n\nEq\n\n\nLParen\n\n\nRParen\n\n\nComma\n\n\nComment(String)\n\n\nLetKeyword\n\n\nInKeyword\n\n\nCaseKeyword\n\n\nOfKeyword\n\n\nWhereKeyword\n\n\nDoKeyword\n\n\nTypeAliasKeyword\n\n\nPortModuleKeyword\n\n\nModuleKeyword\n\n\nIfKeyword\n\n\nThenKeyword\n\n\nElseKeyword\n\n\nImportKeyword\n\n\nExposingKeyword\n\n\nAsKeyword\n\n\nBoolLiteral(Bool)\n\n\nFloatLiteral(Float)\n\n\nIntLiteral(Int)\n\n\nStringLiteral(String)\n\n\nIdentifier(String)\n\n\nArrow\n -&gt;\n\nPipeline\n |&gt;\n\nReverseApply\n &lt;|\n\nCons\n ::\n\nConcat\n ++\n\nPlus\n\n\nMinus\n\n\nMultiply\n\n\nDivide\n\n\nIntDivide\n //\n\nPower\n ^\n\nModulo\n\n\nEqual\n ==\n\nNotEqual\n /=\n\nLessThan\n\n\nGreaterThan\n\n\nLessEqual\n\n\nGreaterEqual\n\n\nLBracket\n\n\nRBracket\n\n\nLBrace\n\n\nRBrace\n\n\nDot\n\n\nDoubleDot\n ..\n\nColon\n\n\nLayoutStart\n Generated { for layout\n\nLayoutEnd\n Generated } for layout\n\nLayoutSemicolon\n Generated ; for layout\n\nSynonyms:\nglelm/elm/lexer.Token\nglelm/elm/lexer Token","url":"glelm/elm/lexer.html#Token"},{"doc":"glelm/elm/lexer","title":"multi_line_comment","content":"pub fn multi_line_comment() -&gt; Matcher(Token, a)\n\nSynonyms:\nglelm/elm/lexer.multi_line_comment\nglelm/elm/lexer multi_line_comment","url":"glelm/elm/lexer.html#multi_line_comment"},{"doc":"glelm/elm/lexer","title":"new","content":"pub fn new() -&gt; Lexer\n\nSynonyms:\nglelm/elm/lexer.new\nglelm/elm/lexer new","url":"glelm/elm/lexer.html#new"},{"doc":"glelm/elm/lexer","title":"run","content":"pub fn run(\n  lx: Lexer,\n  source: String,\n) -&gt; Result(List(Token(glelm/elm/lexer.Token)), Error)\n\nSynonyms:\nglelm/elm/lexer.run\nglelm/elm/lexer run","url":"glelm/elm/lexer.html#run"},{"doc":"glelm/elm/lexer","title":"single_line_comment","content":"pub fn single_line_comment() -&gt; Matcher(Token, a)\n\nSynonyms:\nglelm/elm/lexer.single_line_comment\nglelm/elm/lexer single_line_comment","url":"glelm/elm/lexer.html#single_line_comment"},{"doc":"glelm/elm/lexer","title":"glelm/elm/lexer","content":"","url":"glelm/elm/lexer.html"},{"doc":"glelm/elm/ast","title":"Declaration","content":"pub type Declaration {\n  CustomTypeDeclaration(Type)\n}\n\nCustomTypeDeclaration(Type)\n\n\nSynonyms:\nglelm/elm/ast.Declaration\nglelm/elm/ast Declaration","url":"glelm/elm/ast.html#Declaration"},{"doc":"glelm/elm/ast","title":"Exposing","content":"pub type Exposing {\n  ExposingAll\n  ExposingNothing\n  Explicit(List(TopLevelExpose))\n}\n\nExposingAll\n\n\nExposingNothing\n\n\nExplicit(List(TopLevelExpose))\n\n\nSynonyms:\nglelm/elm/ast.Exposing\nglelm/elm/ast Exposing","url":"glelm/elm/ast.html#Exposing"},{"doc":"glelm/elm/ast","title":"FunctionName","content":"pub type FunctionName {\n  FunctionName(name: String)\n}\n\nFunctionName(name: String)\n\n\nSynonyms:\nglelm/elm/ast.FunctionName\nglelm/elm/ast FunctionName","url":"glelm/elm/ast.html#FunctionName"},{"doc":"glelm/elm/ast","title":"GenericName","content":"pub type GenericName =\n  String\n\n\nSynonyms:\nglelm/elm/ast.GenericName\nglelm/elm/ast GenericName","url":"glelm/elm/ast.html#GenericName"},{"doc":"glelm/elm/ast","title":"Module","content":"pub type Module {\n  Module(\n    name: String,\n    declarations: List(Declaration),\n    exposing: Exposing,\n  )\n}\n\nModule(\n  name: String,\n  declarations: List(Declaration),\n  exposing: Exposing,\n)\n\n\nSynonyms:\nglelm/elm/ast.Module\nglelm/elm/ast Module","url":"glelm/elm/ast.html#Module"},{"doc":"glelm/elm/ast","title":"RecordDefinition","content":"pub type RecordDefinition {\n  RecordDefinition(fields: List(RecordField))\n}\n\nRecordDefinition(fields: List(RecordField))\n\n\nSynonyms:\nglelm/elm/ast.RecordDefinition\nglelm/elm/ast RecordDefinition","url":"glelm/elm/ast.html#RecordDefinition"},{"doc":"glelm/elm/ast","title":"RecordField","content":"pub type RecordField {\n  RecordField(name: RecordFieldName, type_: TypeAnnotation)\n}\n\nRecordField(name: RecordFieldName, type_: TypeAnnotation)\n\n\nSynonyms:\nglelm/elm/ast.RecordField\nglelm/elm/ast RecordField","url":"glelm/elm/ast.html#RecordField"},{"doc":"glelm/elm/ast","title":"RecordFieldName","content":"pub type RecordFieldName =\n  String\n\n\nSynonyms:\nglelm/elm/ast.RecordFieldName\nglelm/elm/ast RecordFieldName","url":"glelm/elm/ast.html#RecordFieldName"},{"doc":"glelm/elm/ast","title":"TopLevelExpose","content":"pub type TopLevelExpose {\n  FunctionExpose(name: FunctionName)\n  TypeOrAliasExpose(name: TypeName, is_opaque: Bool)\n}\n\nFunctionExpose(name: FunctionName)\n\n\nTypeOrAliasExpose(name: TypeName, is_opaque: Bool)\n\n\nSynonyms:\nglelm/elm/ast.TopLevelExpose\nglelm/elm/ast TopLevelExpose","url":"glelm/elm/ast.html#TopLevelExpose"},{"doc":"glelm/elm/ast","title":"Type","content":"pub type Type {\n  Type(\n    name: TypeName,\n    generics: List(GenericName),\n    constructor: List(ValueConstructor),\n  )\n}\n\nType(\n  name: TypeName,\n  generics: List(GenericName),\n  constructor: List(ValueConstructor),\n)\n\n\nSynonyms:\nglelm/elm/ast.Type\nglelm/elm/ast Type","url":"glelm/elm/ast.html#Type"},{"doc":"glelm/elm/ast","title":"TypeAnnotation","content":"pub type TypeAnnotation {\n  GenericType(GenericName)\n  Unit\n  Tupled(List(TypeAnnotation))\n  Typed(TypeName, List(TypeAnnotation))\n  Record(RecordDefinition)\n  GenericRecord(GenericName, RecordDefinition)\n  FunctionType(TypeAnnotation, TypeAnnotation)\n}\n Custom type for different type annotations.\n\n For example:\n\n   GenericType: a\n   Typed: Maybe (Int -&gt; String)\n   Unit: ()\n   Tuples: (a, b, c)\n   Record: { name : String}\n   GenericRecord: { a | name : String}\n   FunctionType: Int -&gt; String\nGenericType(GenericName)\n\n\nUnit\n\n\nTupled(List(TypeAnnotation))\n\n\nTyped(TypeName, List(TypeAnnotation))\n\n\nRecord(RecordDefinition)\n\n\nGenericRecord(GenericName, RecordDefinition)\n\n\nFunctionType(TypeAnnotation, TypeAnnotation)\n\n\nSynonyms:\nglelm/elm/ast.TypeAnnotation\nglelm/elm/ast TypeAnnotation","url":"glelm/elm/ast.html#TypeAnnotation"},{"doc":"glelm/elm/ast","title":"TypeName","content":"pub type TypeName {\n  TypeName(name: String)\n}\n\nTypeName(name: String)\n\n\nSynonyms:\nglelm/elm/ast.TypeName\nglelm/elm/ast TypeName","url":"glelm/elm/ast.html#TypeName"},{"doc":"glelm/elm/ast","title":"ValueConstructor","content":"pub type ValueConstructor {\n  ValueConstructor(\n    name: TypeName,\n    arguments: List(TypeAnnotation),\n  )\n}\n\nValueConstructor(name: TypeName, arguments: List(TypeAnnotation))\n\n\nSynonyms:\nglelm/elm/ast.ValueConstructor\nglelm/elm/ast ValueConstructor","url":"glelm/elm/ast.html#ValueConstructor"},{"doc":"glelm/elm/ast","title":"glelm/elm/ast","content":"","url":"glelm/elm/ast.html"},{"doc":"glelm/elm/parser","title":"Error","content":"pub type Error(ctx) {\n  NoMatchingTokenError(row: Int, col: Int, lexeme: String)\n  ParserError(List(nibble.DeadEnd(lexer.Token, ctx)))\n}\n\nNoMatchingTokenError(row: Int, col: Int, lexeme: String)\n\n\nParserError(List(nibble.DeadEnd(lexer.Token, ctx)))\n\n\nSynonyms:\nglelm/elm/parser.Error\nglelm/elm/parser Error","url":"glelm/elm/parser.html#Error"},{"doc":"glelm/elm/parser","title":"Parser","content":"pub type Parser(a, ctx) =\n  nibble.Parser(a, lexer.Token, ctx)\n\n\nSynonyms:\nglelm/elm/parser.Parser\nglelm/elm/parser Parser","url":"glelm/elm/parser.html#Parser"},{"doc":"glelm/elm/parser","title":"inspect","content":"pub fn inspect(\n  parser: Parser(a, Token, b),\n  msg: String,\n) -&gt; Parser(a, Token, b)\n\nSynonyms:\nglelm/elm/parser.inspect\nglelm/elm/parser inspect","url":"glelm/elm/parser.html#inspect"},{"doc":"glelm/elm/parser","title":"module","content":"pub fn module() -&gt; Parser(Module, Token, a)\n\nSynonyms:\nglelm/elm/parser.module\nglelm/elm/parser module","url":"glelm/elm/parser.html#module"},{"doc":"glelm/elm/parser","title":"parse","content":"pub fn parse(\n  elm_source: String,\n  parser: Parser(a, Token, b),\n) -&gt; Result(a, Error(b))\n\nSynonyms:\nglelm/elm/parser.parse\nglelm/elm/parser parse","url":"glelm/elm/parser.html#parse"},{"doc":"glelm/elm/parser","title":"record_type_annotation","content":"pub fn record_type_annotation() -&gt; Parser(\n  TypeAnnotation,\n  Token,\n  a,\n)\n\nSynonyms:\nglelm/elm/parser.record_type_annotation\nglelm/elm/parser record_type_annotation","url":"glelm/elm/parser.html#record_type_annotation"},{"doc":"glelm/elm/parser","title":"glelm/elm/parser","content":"","url":"glelm/elm/parser.html"},{"doc":"glelm/transpile/context","title":"Context","content":"pub type Context {\n  Context(exposing: Exports)\n}\n\nContext(exposing: Exports)\n\n\nSynonyms:\nglelm/transpile/context.Context\nglelm/transpile/context Context","url":"glelm/transpile/context.html#Context"},{"doc":"glelm/transpile/context","title":"Export","content":"pub type Export {\n  TypeOrAliasExport(is_opaque: Bool)\n  FunctionExport\n}\n\nTypeOrAliasExport(is_opaque: Bool)\n\n\nFunctionExport\n\n\nSynonyms:\nglelm/transpile/context.Export\nglelm/transpile/context Export","url":"glelm/transpile/context.html#Export"},{"doc":"glelm/transpile/context","title":"Exports","content":"pub type Exports {\n  NoPublic\n  AllPublic(dict.Dict(String, Export))\n  SomePublic(dict.Dict(String, Export))\n}\n\nNoPublic\n\n\nAllPublic(dict.Dict(String, Export))\n\n\nSomePublic(dict.Dict(String, Export))\n\n\nSynonyms:\nglelm/transpile/context.Exports\nglelm/transpile/context Exports","url":"glelm/transpile/context.html#Exports"},{"doc":"glelm/transpile/context","title":"Visibility","content":"pub type Visibility {\n  Visibility(is_public: Bool, is_opaque: Bool)\n}\n\nVisibility(is_public: Bool, is_opaque: Bool)\n\n\nSynonyms:\nglelm/transpile/context.Visibility\nglelm/transpile/context Visibility","url":"glelm/transpile/context.html#Visibility"},{"doc":"glelm/transpile/context","title":"handle_custom_type","content":"pub fn handle_custom_type(type_: Type) -&gt; Eval(Nil, a, Context)\n\nSynonyms:\nglelm/transpile/context.handle_custom_type\nglelm/transpile/context handle_custom_type","url":"glelm/transpile/context.html#handle_custom_type"},{"doc":"glelm/transpile/context","title":"handle_exposing","content":"pub fn handle_exposing(\n  exposing: Exposing,\n) -&gt; Eval(Nil, a, Context)\n\nSynonyms:\nglelm/transpile/context.handle_exposing\nglelm/transpile/context handle_exposing","url":"glelm/transpile/context.html#handle_exposing"},{"doc":"glelm/transpile/context","title":"new","content":"pub fn new() -&gt; Context\n\nSynonyms:\nglelm/transpile/context.new\nglelm/transpile/context new","url":"glelm/transpile/context.html#new"},{"doc":"glelm/transpile/context","title":"visibility","content":"pub fn visibility(name: String) -&gt; Eval(Visibility, a, Context)\n\nSynonyms:\nglelm/transpile/context.visibility\nglelm/transpile/context visibility","url":"glelm/transpile/context.html#visibility"},{"doc":"glelm/transpile/context","title":"glelm/transpile/context","content":"","url":"glelm/transpile/context.html"},{"doc":"glelm/transpile","title":"Error","content":"pub type Error {\n  Error(glance.Error)\n}\n\nError(glance.Error)\n\n\nSynonyms:\nglelm/transpile.Error\nglelm/transpile Error","url":"glelm/transpile.html#Error"},{"doc":"glelm/transpile","title":"Transpiler","content":"pub type Transpiler(a) =\n  Eval(a, Error, Context)\n\n\nSynonyms:\nglelm/transpile.Transpiler\nglelm/transpile Transpiler","url":"glelm/transpile.html#Transpiler"},{"doc":"glelm/transpile","title":"custom_type","content":"pub fn custom_type(\n  type_: Type,\n) -&gt; Eval(Definition(CustomType), Error, Context)\n\nSynonyms:\nglelm/transpile.custom_type\nglelm/transpile custom_type","url":"glelm/transpile.html#custom_type"},{"doc":"glelm/transpile","title":"initial_ctx","content":"pub fn initial_ctx() -&gt; Context\n\nSynonyms:\nglelm/transpile.initial_ctx\nglelm/transpile initial_ctx","url":"glelm/transpile.html#initial_ctx"},{"doc":"glelm/transpile","title":"module","content":"pub fn module(\n  elm_ast: Module,\n) -&gt; Eval(glance.Module, Error, Context)\n\nSynonyms:\nglelm/transpile.module\nglelm/transpile module","url":"glelm/transpile.html#module"},{"doc":"glelm/transpile","title":"print","content":"pub fn print(module: Module) -&gt; String\n\nSynonyms:\nglelm/transpile.print\nglelm/transpile print","url":"glelm/transpile.html#print"},{"doc":"glelm/transpile","title":"run","content":"pub fn run(\n  t: Eval(a, Error, Context),\n  s: Context,\n) -&gt; Result(a, Error)\n\nSynonyms:\nglelm/transpile.run\nglelm/transpile run","url":"glelm/transpile.html#run"},{"doc":"glelm/transpile","title":"glelm/transpile","content":"","url":"glelm/transpile.html"},{"doc":"glelm","title":"RuntimeError","content":"pub type RuntimeError(ctx) {\n  ArgumentError(String)\n  ParserError(parser.Error(ctx))\n  TranspileError(transpile.Error)\n  GlanceError(glance.Error)\n  InternalError(String)\n}\n\nArgumentError(String)\n\n\nParserError(parser.Error(ctx))\n\n\nTranspileError(transpile.Error)\n\n\nGlanceError(glance.Error)\n\n\nInternalError(String)\n\n\nSynonyms:\nglelm.RuntimeError\nglelm RuntimeError","url":"glelm.html#RuntimeError"},{"doc":"glelm","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\nglelm.main\nglelm main","url":"glelm.html#main"},{"doc":"glelm","title":"run","content":"pub fn run() -&gt; Result(String, RuntimeError(a))\n\nSynonyms:\nglelm.run\nglelm run","url":"glelm.html#run"},{"doc":"glelm","title":"glelm","content":"","url":"glelm.html"}]);